@@ -4,7 +4,7 @@
 
 ;; Author: Philip Kaludercic <philipk@posteo.net>
 ;; Maintainer: Philip Kaludercic <philipk@posteo.net>
-;; Version: 1.1.0
+;; Version: 1.2.0
 ;; Package-Requires: ((emacs "26.1"))
 ;; Keywords: lisp, local
 ;; URL: https://git.sr.ht/~pkal/setup
@@ -32,10 +32,25 @@
 ;; currently known local macros are documented in the docstring for `setup'.
 
 ;; Examples and extended documentation can be found on Emacs wiki:
-;; https://www.emacswiki.org/emacs/SetupEl
+;; https://www.emacswiki.org/emacs/SetupEl.  Please feel free to
+;; contribute your own local macros or ideas.
 
-;;; News
+;;; News:
 
+;;;; Version 1.2.0
+;;
+;; - Remove `setup-wrap-to-demote-errors' from `setup-modifier-list'
+;; - Pull `setup-expand-local-macros'  back into `setup'
+;; - Let `:with-feature' and `:with-mode' check symbol properties to
+;;   improve context-setting guesses.
+;; - Move macros :hide-mode, :advise, :needs, :if-host and :load-from
+;;   to EmacsWiki.
+;; - Revert the indentation spec change for `setup-define'
+;; - Add :bind-into macro
+;; - Add :ensure key to `setup-define' to replace
+;;   the deprecated setup-ensure-... functions
+;; - Add `setup-bind' macro to simplify context modification.
+;;
 ;;;; Version 1.1.0:
 ;;
 ;; - Fix quoting error in :file-match definition
@@ -60,7 +75,7 @@ is used to retrieve the current context.")
 (defvar setup-attributes '()
   "A list symbols used to store a state during macro processing.
 The list is populated during macro expansion, and may modify the
-behaviour of the functions in `setup-modifier-list'..")
+behaviour of the functions in `setup-modifier-list'.")
 
 (defun setup-wrap-to-catch-quits (body _name)
   "Wrap BODY in a catch block if necessary.
@@ -81,9 +96,7 @@ symbol `without-error-demotion'."
        ,body)))
 
 (defvar setup-modifier-list
-  '(setup-expand-local-macros
-    setup-wrap-to-catch-quits
-    setup-wrap-to-demote-errors)
+  '(setup-wrap-to-catch-quits)
   "List of wrapper functions to be called after macro expansion.
 Each function is invoked by passing the current body and the name
 of the default feature, returning the modified body.")
@@ -93,15 +106,6 @@ of the default feature, returning the modified body.")
 Do not modify this variable by hand.  Instead use
 `setup-define.'")
 
-(defun setup-expand-local-macros (body name)
-  "Expand macros in BODY given by `setup-macros'.
-NAME is a symbol or string designating the default feature."
-  (macroexpand-all
-   (if (assq :with-feature setup-macros)
-       `(:with-feature ,name ,@body)
-     (macroexp-progn body))
-   (append setup-macros macroexpand-all-environment)))
-
 ;;;###autoload
 (defun setup-make-docstring ()
   "Return a docstring for `setup'."
@@ -140,12 +144,17 @@ BODY may contain special forms defined by `setup-define', but
 will otherwise just be evaluated as is.
 NAME may also be a macro, if it can provide a symbol."
   (declare (debug (&rest &or [symbolp sexp] form))
-           (indent defun))
+           (indent 1))
   (when (consp name)
     (push name body)
     (let ((shorthand (get (car name) 'setup-shorthand)))
       (setq name (and shorthand (funcall shorthand name)))))
   (let ((setup-attributes setup-attributes))
+    (setq body (macroexpand-all
+                (if (assq :with-feature setup-macros)
+                    `(:with-feature ,name ,@body)
+                  (macroexp-progn body))
+                setup-macros))
     (dolist (mod-fn setup-modifier-list)
       (setq body (funcall mod-fn body name)))
     body))
@@ -181,8 +190,15 @@ returns a symbol to replace NAME.
 
   :debug SPEC
 A edebug specification, see Info node `(elisp) Specification List'.
-If not given, it is assumed nothing is evaluated."
-  (declare (indent 2))
+If not given, it is assumed nothing is evaluated.
+
+  :ensure SPEC
+
+A list of symbols indicating what kind of argument each parameter
+to FN is.  If the nth parameter is not to be reinterpreted, the
+nth symbol in SPEC should nil.  For key bindings `kbd' and for
+functions `func'.  Any other value is invalid."
+  (declare (indent 1))
   ;; NB.: NAME is not required to by a keyword, even though all macros
   ;;      specified on the next page use keywords.  The rationale for
   ;;      this is currently that there is no clean way to "locally"
@@ -213,6 +229,30 @@ If not given, it is assumed nothing is evaluated."
                          (while args
                            (let ((rest (nthcdr arity args)))
                              (setf (nthcdr arity args) nil)
+                             (let ((ensure-spec (plist-get opts :ensure)))
+                               (when ensure-spec
+                                 (dotimes (i (length args))
+                                   (let ((ensure (nth i ensure-spec))
+                                         (arg (nth i args)))
+                                     (cond
+                                      ((null ensure)) ;Do not modify argument
+                                      ((eq ensure 'kbd)
+                                       (setf (nth i args)
+                                             (cond
+                                              ((stringp arg) (kbd arg))
+                                              ((symbolp arg) `(kbd ,arg))
+                                              (arg))))
+                                      ((eq ensure 'func)
+                                       (setf (nth i args)
+                                             (cond
+                                              ((eq (car-safe arg) 'function)
+                                               arg)
+                                              ((eq (car-safe arg) 'quote)
+                                               `#',(cadr arg))
+                                              ((symbolp arg)
+                                               `#',arg)
+                                              (arg))))
+                                      ((error "Invalid ensure spec %S" ensure)))))))
                              (push (apply fn args) aggr)
                              (setq args rest)))
                          (macroexp-progn (nreverse aggr)))))))
@@ -258,78 +298,83 @@ This must be used in context-setting macros (`:with-feature',
 settings."
   (macroexpand-all (macroexp-progn body) setup-macros))
 
+(defmacro setup-bind (body &rest vars)
+  "Add VARS to `setup-opts' in BODY.
+Each entry in VARS is a list of the form (VAR VAL), comparable to
+`let'.  This macro makes sure that the BODY is expanded correctly
+so that it can make use of the new bindings in VARS."
+  (declare (debug let) (indent 1))
+  ;; The macro modifies VARS in place, inserting unquotes in the right
+  ;; places to convert a `let'-formed list into a alist.  The unquoted
+  ;; values are then handled by the backquote inserted by the macro.
+  ;; The list this generates is destructively concatenated to the
+  ;; beginning of setup-ops, which is safe because backquoting expands
+  ;; to a new list allocation.
+  (dolist (var vars)
+    (setcdr var (list '\, (cadr var))))
+  `(let ((setup-opts (nconc ,(list '\` vars) setup-opts)))
+     (setup-expand ,body)))
+
 (defun setup-quit (&optional return)
   "Generate code to quit evaluation.
 If RETURN is given, throw that value."
   (push 'need-quit setup-attributes)
   `(throw ',(setup-get 'quit) ,return))
 
-(defun setup-ensure-kbd (sexp)
-  "Attempt to return SEXP as a key binding expression."
-  (cond ((stringp sexp) (kbd sexp))
-        ((symbolp sexp) `(kbd ,sexp))
-        (sexp)))
-
-(defun setup-ensure-function (sexp)
-  "Attempt to return SEXP as a quoted function name."
-  (cond ((eq (car-safe sexp) 'function)
-         sexp)
-        ((eq (car-safe sexp) 'quote)
-         `#',(cadr sexp))
-        ((symbolp sexp)
-         `#',sexp)
-        (sexp)))
-
-(defun setup-make-setter (name val old-val-fn wrap-fn)
-  "Convert NAME and VAL into setter code.
-The function OLD-VAL-FN is used to extract the old value of
-VAL.  The function WRAP-FN combines the transformed values of NAME
-and VAL into one s-expression."
-  (cond ((symbolp name) (funcall wrap-fn name val))
-        ((eq (car-safe name) 'append)
-         (funcall wrap-fn
-                  (cadr name)
-                  (let ((sym (gensym)))
-                    `(let ((,sym ,val)
-                           (list ,(funcall old-val-fn (cadr name))))
-                       (if (member ,sym list)
-                           list
-                         (append list (list ,sym)))))))
-        ((eq (car-safe name) 'prepend)
-         (funcall wrap-fn
-                  (cadr name)
-                  (let ((sym (gensym)))
-                    `(let ((,sym ,val)
-                           (list ,(funcall old-val-fn (cadr name))))
-                       (if (member ,sym list)
-                           list
-                         (cons ,sym list))))))
-        ((eq (car-safe name) 'remove)
-         (funcall wrap-fn
-                  (cadr name)
-                  `(remove ,val ,(funcall old-val-fn name))))
-        ((error "Invalid option %S" name))))
+(defun setup-make-setter (old-val-fn wrap-fn)
+  "Return a macro function to generate a setter.
+The function OLD-VAL-FN is used to extract the old value of VAL.
+The function WRAP-FN combines the transformed values of NAME and
+VAL into one s-expression."
+  (lambda (name val)
+    (cond ((symbolp name) (funcall wrap-fn name val))
+          ((eq (car-safe name) 'append)
+           (funcall wrap-fn
+                    (cadr name)
+                    (let ((sym (gensym)))
+                      `(let ((,sym ,val)
+                             (list ,(funcall old-val-fn (cadr name))))
+                         (if (member ,sym list)
+                             list
+                           (append list (list ,sym)))))))
+          ((eq (car-safe name) 'prepend)
+           (funcall wrap-fn
+                    (cadr name)
+                    (let ((sym (gensym)))
+                      `(let ((,sym ,val)
+                             (list ,(funcall old-val-fn (cadr name))))
+                         (if (member ,sym list)
+                             list
+                           (cons ,sym list))))))
+          ((eq (car-safe name) 'remove)
+           (funcall wrap-fn
+                    (cadr name)
+                    `(remove ,val ,(funcall old-val-fn (cadr name)))))
+          ((error "Invalid option %S" name)))))
 
 
 ;;; Default local macros definitions
 
 (setup-define :with-feature
-    (lambda (features &rest body)
-      (let (bodies)
-        (dolist (feature (if (listp features) features (list features)))
-          (push (if feature
-                    (let* ((mode (if (string-match-p "-mode\\'" (symbol-name feature))
-                                     feature
-                                   (intern (format "%s-mode" feature))))
-                           (setup-opts `((feature . ,feature)
-                                         (mode . ,mode)
-                                         (hook . ,(intern (format "%s-hook" mode)))
-                                         (map . ,(intern (format "%s-map" mode)))
-                                         ,@setup-opts)))
-                      (setup-expand body))
-                  body)
-                bodies))
-        (macroexp-progn (if features (nreverse bodies) body))))
+  (lambda (features &rest body)
+    (let (bodies)
+      (dolist (feature (if (listp features) features (list features)))
+        (push (if feature
+                  (let ((mode (if (string-match-p "-mode\\'" (symbol-name feature))
+                                  feature
+                                (intern (format "%s-mode" feature)))))
+                    (setup-bind body
+                      (feature feature)
+                      (mode (or (get features 'setup-mode) mode))
+                      (hook (or (get features 'setup-hook)
+                                (get mode 'setup-hook)
+                                (intern (format "%s-hook" mode))))
+                      (map (or (get features 'setup-map)
+                               (get mode 'setup-map)
+                               (intern (format "%s-map" mode))))))
+                body)
+              bodies))
+      (macroexp-progn (if features (nreverse bodies) body))))
   :documentation "Change the FEATURE that BODY is configuring.
 This macro also declares a current mode by appending \"-mode\" to
 FEATURE, unless it already ends with \"-mode\".
@@ -338,53 +383,52 @@ If FEATURE is a list, apply BODY to all elements of FEATURE."
   :indent 1)
 
 (setup-define :with-mode
-    (lambda (modes &rest body)
-      (let (bodies)
-        (dolist (mode (if (listp modes) modes (list modes)))
-          (push (let ((setup-opts `((mode . ,mode)
-                                    (hook . ,(intern (format "%s-hook" mode)))
-                                    (map . ,(intern (format "%s-map" mode)))
-                                    ,@setup-opts)))
-                  (setup-expand body))
-                bodies))
-        (macroexp-progn (nreverse bodies))))
+  (lambda (modes &rest body)
+    (let (bodies)
+      (dolist (mode (if (listp modes) modes (list modes)))
+        (push (setup-bind body
+                (mode mode)
+                (hook (or (get mode 'setup-hook)
+                          (intern (format "%s-hook" mode))))
+                (map (or (get mode 'setup-map)
+                         (intern (format "%s-map" mode)))))
+              bodies))
+      (macroexp-progn (nreverse bodies))))
   :documentation "Change the MODE that BODY is configuring.
 If MODE is a list, apply BODY to all elements of MODE."
   :debug '(sexp setup)
   :indent 1)
 
 (setup-define :with-map
-    (lambda (maps &rest body)
-      (let (bodies)
-        (dolist (map (if (listp maps) maps (list maps)))
-          (push (let ((setup-opts (cons `(map . ,map) setup-opts)))
-                  (setup-expand body))
-                bodies))
-        (macroexp-progn (nreverse bodies))))
+  (lambda (maps &rest body)
+    (let (bodies)
+      (dolist (map (if (listp maps) maps (list maps)))
+        (push (setup-bind body (map map))
+              bodies))
+      (macroexp-progn (nreverse bodies))))
   :documentation "Change the MAP that BODY will bind to.
 If MAP is a list, apply BODY to all elements of MAP."
   :debug '(sexp setup)
   :indent 1)
 
 (setup-define :with-hook
-    (lambda (hooks &rest body)
-      (let (bodies)
-        (dolist (hook (if (listp hooks) hooks (list hooks)))
-          (push (let ((setup-opts (cons `(hook . ,hook) setup-opts)))
-                  (setup-expand body))
-                bodies))
-        (macroexp-progn (nreverse bodies))))
+  (lambda (hooks &rest body)
+    (let (bodies)
+      (dolist (hook (if (listp hooks) hooks (list hooks)))
+        (push (setup-bind body (hook hook))
+              bodies))
+      (macroexp-progn (nreverse bodies))))
   :documentation "Change the HOOK that BODY will use.
 If HOOK is a list, apply BODY to all elements of HOOK."
   :debug '(sexp setup)
   :indent 1)
 
 (setup-define :package
-    (lambda (package)
-      `(unless (package-installed-p ',package)
-         (unless (memq ',package package-archive-contents)
-           (package-refresh-contents))
-         (package-install ',package)))
+  (lambda (package)
+    `(unless (package-installed-p ',package)
+       (unless (memq ',package package-archive-contents)
+         (package-refresh-contents))
+       (package-install ',package)))
   :documentation "Install PACKAGE if it hasn't been installed yet.
 This macro can be used as NAME, and it will replace itself with
 the first PACKAGE."
@@ -392,9 +436,9 @@ the first PACKAGE."
   :shorthand #'cadr)
 
 (setup-define :require
-    (lambda (feature)
-      `(unless (require ',feature nil t)
-         ,(setup-quit)))
+  (lambda (feature)
+    `(unless (require ',feature nil t)
+       ,(setup-quit)))
   :documentation "Try to require FEATURE, or stop evaluating body.
 This macro can be used as NAME, and it will replace itself with
 the first FEATURE."
@@ -402,76 +446,80 @@ the first FEATURE."
   :shorthand #'cadr)
 
 (setup-define :global
-    (lambda (key command)
-      `(global-set-key
-        ,(setup-ensure-kbd key)
-        ,(setup-ensure-function command)))
+  (lambda (key command)
+    `(global-set-key ,key ,command))
   :documentation "Globally bind KEY to COMMAND."
   :debug '(form sexp)
+  :ensure '(kbd func)
   :repeatable t)
 
 (setup-define :bind
-    (lambda (key command)
-      `(define-key ,(setup-get 'map)
-         ,(setup-ensure-kbd key)
-         ,(setup-ensure-function command)))
+  (lambda (key command)
+    `(define-key ,(setup-get 'map) ,key ,command))
   :documentation "Bind KEY to COMMAND in current map."
   :after-loaded t
   :debug '(form sexp)
+  :ensure '(kbd func)
   :repeatable t)
 
 (setup-define :unbind
-    (lambda (key)
-      `(define-key ,(setup-get 'map)
-         ,(setup-ensure-kbd key)
-         nil))
+  (lambda (key)
+    `(define-key ,(setup-get 'map) ,key nil))
   :documentation "Unbind KEY in current map."
   :after-loaded t
   :debug '(form)
+  :ensure '(kbd)
   :repeatable t)
 
 (setup-define :rebind
-    (lambda (key command)
-      `(progn
-         (dolist (key (where-is-internal ',command ,(setup-get 'map)))
-           (define-key ,(setup-get 'map) key nil))
-         (define-key ,(setup-get 'map)
-           ,(setup-ensure-kbd key)
-           ,(setup-ensure-function command))))
+  (lambda (key command)
+    `(progn
+       (dolist (key (where-is-internal ',command ,(setup-get 'map)))
+         (define-key ,(setup-get 'map) ,key nil))
+       (define-key ,(setup-get 'map) ,key ,command)))
   :documentation "Unbind the current key for COMMAND, and bind it to KEY."
   :after-loaded t
   :debug '(form sexp)
+  :ensure '(kbd func)
   :repeatable t)
 
+(setup-define :bind-into
+  (lambda (feature &rest rest)
+    `(:with-feature ,feature (:bind ,@rest)))
+  :documentation "Bind into keys into the map of FEATURE.
+The arguments REST are handled as by `:bind'."
+  :debug '(sexp &rest form sexp)
+  :indent 1)
+
 (setup-define :hook
-    (lambda (function)
-      `(add-hook ',(setup-get 'hook) ,(setup-ensure-function function)))
+  (lambda (function)
+    `(add-hook ',(setup-get 'hook) ,function))
   :documentation "Add FUNCTION to current hook."
+  :ensure '(func)
   :repeatable t)
 
 (setup-define :hook-into
-    (lambda (mode)
-      `(add-hook ',(let ((name (symbol-name mode)))
-                     (if (string-match-p "-hook\\'" name)
-                         mode
-                       (intern (concat name "-hook"))))
-                 ,(setup-ensure-function (setup-get 'mode))))
+  (lambda (mode)
+    `(add-hook ',(let ((name (symbol-name mode)))
+                   (if (string-match-p "-hook\\'" name)
+                       mode
+                     (intern (concat name "-hook"))))
+               #',(setup-get 'mode)))
   :documentation "Add current mode to HOOK."
   :repeatable t)
 
 (setup-define :option
-    (lambda (name val)
-      (setup-make-setter
-       name val
-       (lambda (name)
-         `(funcall (or (get ',name 'custom-get)
-                       #'symbol-value)
-                   ',name))
-       (lambda (name val)
-         `(progn
-            (custom-load-symbol ',name)
-            (funcall (or (get ',name 'custom-set) #'set-default)
-                     ',name ,val)))))
+  (setup-make-setter
+   (lambda (name)
+     `(funcall (or (get ',name 'custom-get)
+                   #'symbol-value)
+               ',name))
+   (lambda (name val)
+     `(progn
+        (custom-load-symbol ',name)
+        (funcall (or (get ',name 'custom-set) #'set-default)
+                 ',name ,val))))
+
   :documentation "Set the option NAME to VAL.
 NAME may be a symbol, or a cons-cell.  If NAME is a cons-cell, it
 will use the car value to modify the behaviour.  These forms are
@@ -495,22 +543,12 @@ therefore not be stored in `custom-set-variables' blocks."
   :debug '(sexp form)
   :repeatable t)
 
-(setup-define :hide-mode
-    (lambda ()
-      `(setq minor-mode-alist
-             (delq (assq ,(setup-get 'mode) minor-mode-alist)
-                   minor-mode-alist)))
-  :documentation "Hide the mode-line lighter of the current mode."
-  :after-loaded t)
-
 (setup-define :local-set
-    (lambda (name val)
-      (setup-make-setter
-       name val
-       (lambda (name)
-         (if (consp name) (cadr name) name))
-       (lambda (name val)
-         `(add-hook ',(setup-get 'hook) (lambda () (setq-local ,name ,val))))))
+  (setup-make-setter
+   (lambda (name)
+     (if (consp name) (cadr name) name))
+   (lambda (name val)
+     `(add-hook ',(setup-get 'hook) (lambda () (setq-local ,name ,val)))))
   :documentation "Set the value of NAME to VAL in buffers of the current mode.
 NAME may be a symbol, or a cons-cell.  If NAME is a cons-cell, it
 will use the car value to modify the behaviour. These forms are
@@ -529,41 +567,26 @@ supported:
   :repeatable t)
 
 (setup-define :local-hook
-    (lambda (hook function)
-      `(add-hook ',(setup-get 'hook)
-                 (lambda ()
-                   (add-hook ',hook ,(setup-ensure-function function) nil t))))
+  (lambda (hook function)
+    `(add-hook ',(setup-get 'hook)
+               (lambda ()
+                 (add-hook ',hook ,function nil t))))
   :documentation "Add FUNCTION to HOOK only in buffers of the current mode."
   :debug '(symbolp sexp)
-  :repeatable t)
-
-(setup-define :advise
-    (lambda (symbol where function)
-      `(advice-add ',symbol ,where ,(setup-ensure-function function)))
-  :documentation "Add a piece of advice on a function.
-See `advice-add' for more details."
-  :after-loaded t
-  :debug '(sexp sexp function-form)
+  :ensure '(nil func)
   :repeatable t)
 
 (setup-define :also-load
-    (lambda (feature)
-      `(require ',feature))
+  (lambda (feature)
+    `(require ',feature))
   :documentation "Load FEATURE with the current body."
   :after-loaded t
   :repeatable t)
 
-(setup-define :needs
-    (lambda (executable)
-      `(unless (executable-find ,executable)
-         ,(setup-quit)))
-  :documentation "If EXECUTABLE is not in the path, stop here."
-  :repeatable 1)
-
 (setup-define :if-package
-    (lambda (package)
-      `(unless (package-installed-p ',package)
-         ,(setup-quit)))
+  (lambda (package)
+    `(unless (package-installed-p ',package)
+       ,(setup-quit)))
   :documentation "If package is not installed, stop evaluating the body.
 This macro can be used as NAME, and it will replace itself with
 the first PACKAGE."
@@ -571,54 +594,33 @@ the first PACKAGE."
   :shorthand #'cadr)
 
 (setup-define :if-feature
-    (lambda (feature)
-      `(unless (featurep ',feature)
-         ,(setup-quit)))
+  (lambda (feature)
+    `(unless (featurep ',feature)
+       ,(setup-quit)))
   :documentation "If FEATURE is not available, stop evaluating the body.
 This macro can be used as NAME, and it will replace itself with
 the first PACKAGE."
   :repeatable t
   :shorthand #'cadr)
 
-(setup-define :if-host
-    (lambda (hostname)
-      `(unless (string= (system-name) ,hostname)
-         ,(setup-quit)))
-  :documentation "If HOSTNAME is not the current hostname, stop evaluating form.")
-
 (setup-define :only-if
-    (lambda (condition)
-      `(unless ,condition
-         ,(setup-quit)))
-  :documentation "If CONDITION is non-nil, stop evaluating the body."
+  (lambda (condition)
+    `(unless ,condition
+       ,(setup-quit)))
+  :documentation "If CONDITION evaluates to nil, stop evaluating the body."
   :debug '(form)
   :repeatable t)
 
-(setup-define :load-from
-    (lambda (path)
-      `(let ((path* (expand-file-name ,path)))
-         (if (file-exists-p path*)
-             (add-to-list 'load-path path*)
-           ,(setup-quit))))
-  :documentation "Add PATH to load path.
-This macro can be used as NAME, and it will replace itself with
-the nondirectory part of PATH.
-If PATH does not exist, abort the evaluation."
-  :shorthand (lambda (args)
-               (intern
-                (file-name-nondirectory
-                 (directory-file-name (cadr args))))))
-
 (setup-define :file-match
-    (lambda (pat)
-      `(add-to-list 'auto-mode-alist (cons ,pat ',(setup-get 'mode))))
+  (lambda (pat)
+    `(add-to-list 'auto-mode-alist (cons ,pat ',(setup-get 'mode))))
   :documentation "Associate the current mode with files that match PAT."
   :debug '(form)
   :repeatable t)
 
 (setup-define :when-loaded
-    (lambda (&rest body)
-      (macroexp-progn body))
+  (lambda (&rest body)
+    (macroexp-progn body))
   :documentation "Evaluate BODY after the current feature has been loaded.
 Avoid using this macro whenever possible, and
 instead choose a more specialized alternative or write one
@@ -627,6 +629,27 @@ yourself."
   :after-loaded t
   :indent 0)
 
+
+;;; Obsoleted code
+
+(defun setup-ensure-kbd (sexp)
+  "Attempt to return SEXP as a key binding expression."
+  (cond ((stringp sexp) (kbd sexp))
+        ((symbolp sexp) `(kbd ,sexp))
+        (sexp)))
+(make-obsolete 'setup-ensure-kbd "Use :ensure keyword instead" "1.2.0")
+
+(defun setup-ensure-function (sexp)
+  "Attempt to return SEXP as a quoted function name."
+  (cond ((eq (car-safe sexp) 'function)
+         sexp)
+        ((eq (car-safe sexp) 'quote)
+         `#',(cadr sexp))
+        ((symbolp sexp)
+         `#',sexp)
+        (sexp)))
+(make-obsolete 'setup-ensure-function "Use :ensure keyword instead" "1.2.0")
+
 (provide 'setup)
 
 ;;; setup.el ends here
